.nr PO 1i
.po 1i
.nr LL 6.5i
.ll 6.5i
.ds CH
.DS C
.ft B
Managing an Ethernet Installation:
Case Studies from the Front Lines
.ft I
.sp
Chris Johnson
.ft R
Northeastern University
.DE
.2C
.NH
Intro
.PP
This article deals with some of the problems encountered with the
Ethernet based campus network at Northeastern University in Boston,
Massachusetts
.PP
When this network was first built, it was done at a time when
Ethernet version 1.0 was out, Ethernet version 2.0 had just come out and
IEEE 802.3 wasn't yet a standard.  Therefore there were three different
kinds of Ethernet hardware on the market.  Vendors were trying to sell
their V1.0 equipment as well as the V2.0 equipment.  Many of them were
also starting to sell 802.3 hardware based on the then proposed IEEE
standard.
.PP
It should also be noted that the people who instigated this network
were not very experienced in such things as LANs.  Some of them had
dealt with wide area networks but none had dealt with an Ethernet based
LAN and they were not network engineers.
.PP
Added together these facts made for an interesting combination of
hardware and growth patterns on the campus network.
.PP
The campus network started as a collaboration between three
departments and originally connected three buildings.  Over the last few
years it grew.  In fact it grew rather explosively from a small informal
network between three departments into a campus LAN.  Of course
all things start small, however this grew faster than it was
planned.  All the problems associated with rapid and improperly thought
out growth logically followed.
.PP
About three years ago the network consisted of mostly terminal
servers and a few hosts.  Parts of the network were down more than up in
the sense that parts would magically vanish from the rest.  Connectivity
was something one dreamed about when one was not running around trying to
reboot terminal server boxes.
.PP
At this point our new network coordinator and I started reading the
three Ethernet specifications in excruciating detail and examining the
actual hardware in the field.  What was found was a collection of
standards and broken rules.  It soon became apparent why the network had
so many problems.
.PP
When our hardware person and I started investigating, we found that
we had the following problems:
.RS
.IP \(bu
Ethernet co-ax runs of well over the 500 m segment length as well
as more than the number of repeaters allowed by any of the three
standards.  No LAN bridges on an Ethernet that desperately needed
them.
.IP \(bu
Tranceivers, transceiver cables and multiport units of all three
standard types indiscriminately intermixed as well as heartbeat
problems.
.IP \(bu
Tight loops and kinks in the co-ax, mechanically insecure
hardware and indiscriminate use of barrel connectors.
.IP \(bu
Improperly cascaded multiport units for existing conditions.
.IP \(bu
Lack of documentation.
.RE
.PP
In short, a mess.  The miracle is that any of it worked at all.
Of course, it worked very poorly.  We had too many collisions and
alignment errors, terminal servers wouldn't boot, connections were flakey
at best.
.PP
What I had intended to come up with was a list of things to do and
a list of things not to do.  The whole thing boiled down to:
.RS
.IP \(bu
Pick a well known standard.
.IP \(bu
Understand that standard.
.IP \(bu
Follow that standard.
.IP \(bu
Have bolt cutters at the ready for any body who touches your
network without your permission.
.RE
.LP
And that about covered it.
.PP
We are now going around straightening out or cutting out the
kinks,  uncoiling the tight coils, replacing segments where necessary,
upgrading to 802.3, putting in LAN bridges (and a router), fixing
cables, etc.  We're following the rules in other words.  It's going to
take time and should have been done right to begin with.
.PP
I'm starting a museum of parts which we remove from the net that
have caused problems.  These will, hopefully, be built into an exhibit
with documentation for each segment explaining where it came from, it's
environment, and if possible what errors it may have contributed to the
network.  Unfortunately there are some people's machines contributing
errors and I can't pull them.
.PP
What follows is a list of cases that were found and corrected.  With
each correction and proper application of the rules the number, type and
frequency of problems diminished, often dramatically.
.NH
Case studies
.NH 2
Length considerations.
.PP
All three standards clearly state the maximum length of a single
segment of co-ax cable as 500 meters.  Several of the segments in
operation were longer.  These segments had many terminal server boxes on
them that were very hard to boot off of the central boot servers and
when up they did not stay up long.  Connection time-outs and the
inability to make a connection were also a common occurrence.
.PP
Splitting these long runs into electrically isolated segments less
than the maximum length did wonders.  The segments were split using
either repeaters or bridges.  We also tried very hard to follow 802.3
when using the repeaters.  In some cases we were limited in what we
could do with this kind of repair.  The network was not funded by the
university.  Any money put towards it had to come from our departmental
budget.  There wasn't much of that available.
.PP
There are rules defined for the use of repeaters.  The number of
repeaters between any two stations is a primary limitation.  This has to
do with repeaters adding effective "length" to a segment.  The V1.0,
V2.0, and 802.3 specifications allow a maximum of two repeaters between
any two stations on co-ax segments.  The 802.3 rules allow five segments, two
of which are fiber optic, but still only three co-ax segments and two
co-ax repeaters.  In several cases there were at least five or six
repeaters between a number of stations. Some of them were half-repeater
fiber optic links but there were still too many.
.PP
The repeater count problems caused disturbances similar to the over
length difficulties.  Also, the repeaters on these segments were subject
to going crazy, on a semi-regular basis, by causing jabber, collisions
because of late packets, and jammed indications.  They would have to be
power cycled in order to be reset.
.PP
The introduction of LAN bridges reduced the number of repeaters.  This
got us under the limit of the number of repeaters in a chain.  It also
left us with fewer repeaters and hence fewer that could fail or require
power cycling.  The number of network disruptions as a result of having
too many repeaters went down.
.PP
In general node connectivity increased, terminal server boxes
started booting easier as well as staying up longer.  The number of
connection time-outs also dropped to zero locally when the repeater
problem was reduced.
.PP
We are still abusing the repeater rules in some cases but the
abuses are going down and network reliability is continuously going up.
.NH 2
Mixed standard hardware.
.PP
Since the network was built at a time when three standards were in
force, there were, and still are, three standards worth of transceiver,
transceiver cable and multiport units on the network.  One problem we
ran into was the SQE or Signal Quality Error test, more commonly called
heartbeat.
.PP
SQE was not defined in Ethernet V1.0.  It was defined in Ethernet
V2.0  Its name was changed in 802.3.  There was much confusion about
this on our network since it was a mixed standard network.
.PP
Some of our host Ethernet controller boards require that heartbeat
be present.  Without heartbeat they count up errors very fast.  From the
point of view of nodes with these controllers, and for reason which are
still not clear, the network was very intermittent.  Our Ethernet
monitor was quite happy with the state of the network as were most other
nodes.
.PP
While reading the three standards, digging under the floor, and
reading copious amounts of vendor literature, we determined the problem
to be mixed standards.  This shows the importance of sticking to one
standard.  Transceiver cable had been ordered and built without regard
to the standards, simply copying what was found out on the network.  If
what they saw was one standard then they copied that. If it was another,
they copied that.  Wherever the standards matched, everything worked.
Where the standards were mixed we saw errors and intermittent network
access on some nodes.  Very confusing indeed since we didn't know what
we had under the floor for cabling at the time.
.PP
There are some differences between transceiver cable grounding
standards.  This plus the fact that we later found (and had to work to
convince the university electrician) grounding problems in the computer
room where most of the network hardware was located could easily have
caused more problems.  We had some many that it was difficult to tell.
.PP
We are slowly replacing everything with 802.3 compliant equipment.
The nodes that were unhappy campers are now much happier living with
the proper use of heartbeat than without it.  Having transceiver cables
that are all the same 802.3 standard and a much better grounded computer
room has helped as well.
.NH 2
Tight loops and kinks in the co-ax, mechanically insecure
hardware and indiscriminate use of barrel connectors.
.PP
Kinks and loops are another issue.  There is a minimum radius of
bend for co-ax of two feet.  There were and probably still are segments
with both tight bends and kinks.  These kinks can best be described as
knuckles which is what they look like.  The loops and bends are of
varying size but range from less than a few inches (with a barrel
connecter between) to about a foot to a foot and half.
.PP
Our communications rack, where all the local Ethernet equipment is
kept, is something one walks behind very carefully.  We again have mixed
standards with regard to connections between transceiver cables and
multi-port units or repeaters and bridges.  We have non-standard screw-in
D plug housings mated to pseudo-standard compliant slide type recepticles.
There is, therefore, nothing holding them in except friction and prayer.
It is unfortunate that although there are standards for the slide type
connectors, vendors don't follow them very closely.  Thus the slides are
ignored in many shops.
.PP
The standards have rules for when, were, and how to use barrel
connectors.  We had and maybe still have segments that used barrel
connectors in very improper ways.
.PP
When using barrel connectors, the co-ax on either side should be of
the same manufacturer and lot number as stated in 802.3.  Also, you want
to use as few barrels as possible.  Barrel connectors are quite visible
when a co-ax segment is shot with a TDR.  If the co-ax on either side is
of different lot numbers or manufacturers it is possible to see this as
well.  Our intrepid tech crew has found segments with four barrel
connectors, one of which was near a tight loop at the end of a cable
with another one in the middle of the loop.  Barrel connectors are also
used as "gender-benders" for terminators on co-ax.  Since there are
terminators of the proper gender, this is a silly thing to do and only
adds to the problems.  The original problem that propagated through the
network was that real network people did not build the network.
.PP
The kinks, loops, and barrel connectors all cause reflections and
are all clearly visible on a TDR scope.  Reflections look like collisions
to an Ethernet segment.  They can also look like alignment errors.  Our
Ethernet monitor counts both.  The bad news is that it counted quite often.
The good news is that it now counts a lot less often since we have been
replacing cable.
.PP
Co-ax in general, whether Ethernet or not, doesn't like to be moved
around much.  It really wants a fixed home where it can live in relative
peace.  Barrel connectors are much the same way.  When used, they should
be left alone and not plugged and un-plugged.  Although much of our
cable is fixed, there are some parts that are subject to mechanical
strain and environmentally insecure circumstances.
.PP
I have a barrel connector in my office with a broken center post
sleeve.  We still don't know how that happened.  The segment it was in
was subject to errors as well.  The bad barrel connector was removed.
Unfortunately this wasn't the only problem on that segment.
.PP
A co-ax  cable in the Chemistry building ran through a room next to
a chemical closet.  This segment also had a tap removed. The tap hole
had not been filled or covered.  Chemical closets are interesting
places, no matter how clean they are kept there are still things in the
air.  This room next to the closet stores glassware and it also is not
the cleanest place in the world.  The particular segment is no longer in
use.  This is good since I saw green where copper should have been in
the unfilled tap hole site.  Removing this segment dropped collisions
and alignment errors from roughly 300 to 400 a day down to 1 or 2 a day.
To be honest, I don't know for sure that the corrosion actually caused a
problem.  There are many other things that could have gone wrong.  The
corrosion is the only thing we noticed on this segment however.
.PP
It has also been known to rain in the core of the Chemistry
building, right on our equipment.  This of course is not really great
for either equipment or tech crew people.
.PP
Building cores and ceilings also have other living things in them
besides the odd tech crew member.  Traces have been found of chewed
cables and there are the occasional strange noises and moving shadows.
.PP
Early on in the review processes we found a couple of segments under
our computer room floor that were tightly looped.  It was heartening to
watch the number of collisions and alignment errors drop by many
hundreds while we straightened these pieces out.
.PP
In one series of buildings, when those who installed the cable were
pulling it, they didn't remove it correctly from the spool.  Rather than
unwinding it, it is quite evident that they did something else.  We have
many short segments down the length of this run that were given the term
knuckles by the tech crew that replaced the segment.  That is exactly
what they look like.  They are very sharp bends over small distances,
say four inches or less.  Another piece of this same cable has a barrel
connector in it which has on either side two three inch diameter 180
degree wrap arounds which go in opposite directions.  The whole piece was
wire tied to a pipe.  These two examples are of course well below the
minimum radius of bend in the any of the three Ethernet specifications.
.PP
At the end of this segment was a tight coil of about 30 feet of
cable with two more barrel connectors in it.  This coil was about one
foot across, also below the minimum radius of bend limit.
.PP
When the tech crew shot this segment with a TDR it looked like a
sky-line of the Rocky Mountains.
.PP
Replacing this segment lowered the alignment error rate from the
approximately 400 per day to around two to four.  The segment was
replaced by the book.  It was fully unrolled, not bent, and had no coils
or unnecessary barrel connectors.
.PP
Another segment, which doesn't belong to us was causing 14000
alignment errors a day.  This segment was our least reliable and often
went out completely.  The department who owns this segment claims that
it is within the specifications.
.PP
Our tech crew found that this segment was coiled under a two foot
floor tile and had Ethernet V1.0, V2.0 and 802.3 taps in it.  The coil
was a wonderful piece of spiro-form with varying radius ranging from two
foot to about six inches or less.  This interesting chunk of co-ax also
had two un-filled and very open ex-tap wholes in it.  It was resting
under a computer room floor tile with filthy air being pumped through it.
Removale of this segment cause the alignment error records to drop from
14000 a day to just three or four.
.NH 2
Improperly cascaded multiport units for existing conditions.
.PP
There should be rules for the cascading of multiport transceiver
units.  Limits as to how deep the cascade can go depend on whether
or not the root multiport is connected to a co-ax segment.  Two layers
deep is a good limit if the root unit goes to a co-ax cable and this
many even be pushing it.  We found cascaded multiport transceiver units
several layers deep and connected to co-ax.
.PP
Whether or not the above functioned was problematical.  Since it
was being done without knowledge of the standards or much knowledge of
things such as signal delay, it was built hit or miss and functioned much
the same way.
.PP
Clearing this up and redistribution the multiport units solved a
lot of problems.  The system now works the same way every time which was
not true before the rearranging.
.NH 2
Lack of documentation.
.PP
Many of the problems on the net could have been avoided by having
documentation stating what was where and what kind it was.  We still do
not have much.  Most of it is either in various heads or you go look.
.PP
Certainly it would be much easier to find everything if we had
geography data on where things are.  As it is now our intrepid techies
have to walk, crawl, and occasionally fly, under, over, and in places
people should not be in order to find things.  An example here is a
building core with no lights and holes in the floor.  I sometimes think
our tech crew should get hazardous duty pay.
